/* @(#)dumpinfo.c	1.3 9/4/92 */
/***************************************************************************/
/*
        Name: dumpinfo.c
 Description: Dumps the contents of an INFO file to a system file.  Dumped
                file does not contain any newlines.  Output is directed to 
                stdout.
     History: 04/92  Todd Stellhorn  - original coding
              09/92  Todd Stellhorn  - flush stdout
 */
/***************************************************************************/

#include <stdio.h>
#include <string.h>
/* #include "infotools.h" /*

/***************************************************************************/

#define USAGE "usage: dumpinfo <INFO_File>" 

short int OutputRecord();

/***************************************************************************/

int main( argc, argv )

int argc;
char **argv;

/***************************************************************************/
/*
       Name: main
    Purpose: Main dumpinfo function.
     Inputs: INFO_File - name of the input INFO file
    Outputs: Function ( int ) - function status
             Dumped data directed to stdout.
    History: 04/92  Todd Stellhorn  - original coding
 */
{
      char              *InfoFilePath;
      InfoFile          *FilePtr;
      TemplateList      *Template;
      long int           i;

/* check args */

      if( argc != 2 ) FatalError( USAGE );

      InfoFilePath = argv[1];

/* open file */

      if(( FilePtr = InfoOpenFile( InfoFilePath, InfoREAD )) ==  
           ( InfoFile * ) NULL )
           FatalError( "Cannot open INFO file." );

/* fetch file template */

      if(( Template = InfoGetTemplate( FilePtr )) == ( TemplateList * ) NULL )
           FatalError( "Error reading INFO file template." );

/* for each record */

      for( i = 1; i <= FilePtr->NumberRecords; i++ )
      {
           if( ! InfoReadRecord( FilePtr, i ))
                FatalError( "Error reading record." );

           if( ! OutputRecord( FilePtr, Template ))
                FatalError( "Error outputing record." );
      }

/* close up shop */

      if( ! InfoCloseFile( FilePtr ))
           FatalError( "Error closing INFO file." );

      (void) fprintf( stdout, OK_STRING );
      (void) fflush( stdout );

      return( 0 );
}                                           /* end of main                 */


/***************************************************************************/

short int OutputRecord( FilePtr, Template )

InfoFile *FilePtr;
TemplateList *Template;

/***************************************************************************/
/*
       Name: OutputRecord
    Purpose: Outputs all data values for a single INFO record.
     Inputs: FilePtr ( InfoFile * ) INFO file pointer
             Template ( TemplateList * ) INFO file template 
    Outputs: Function ( short int ) - function status
             Output record is directed to stdout.
    History: 04/92  Todd Stellhorn  - original coding
 */
{
      char              RetString[4096];
      double            DRetVal;
      char              fmtbuf[128];
      short int         i;

/* decode record and output */

      for( i = 0; i < FilePtr->NumberItems; i++ )
      {
           if( ! InfoDecode( FilePtr, &Template[i], RetString, &DRetVal ))
                return( 0 );

           switch( Template[i].ItemType )
           {
                case INFO_DATE_TYPE         :  
                     return( 0 );

                case INFO_CHARACTER_TYPE    :
                     (void) fprintf( stdout, RetString );
                     break;

                case INFO_INTEGER_TYPE      :
                     (void) sprintf( fmtbuf, "%%%dld", Template[i].OutputWidth );
                     (void) fprintf( stdout, fmtbuf, ( long int ) DRetVal );
                     break;

                case INFO_NUMBER_TYPE       : 
                     (void) sprintf( fmtbuf, "%%%d.%dlf", Template[i].OutputWidth,
                          Template[i].NumberDecimals );
                     (void) fprintf( stdout, fmtbuf, DRetVal );
                     break;

                case INFO_BINARY_TYPE       :
                     (void) sprintf( fmtbuf, "%%%dld", Template[i].OutputWidth );
                     (void) fprintf( stdout, fmtbuf, ( long int ) DRetVal );
                     break;

                case INFO_FLOATING_TYPE     :
                     (void) sprintf( fmtbuf, "%%%d.%dlf", Template[i].OutputWidth,
                          Template[i].NumberDecimals );
                     (void) fprintf( stdout, fmtbuf, DRetVal );
                     break;

                case INFO_LEADFILL_TYPE     :
                case INFO_PACKED_TYPE       :
                case INFO_ZEROFILL_TYPE     :
                case INFO_OVERPUNCH_TYPE    :
                case INFO_TRAILINGSIGN_TYPE :
                case INFO_RECNO_TYPE        :
                default                     :
                     return( 0 );

           }
      }

      return( 1 );  /* valid return */
}                                           /* end of OutputRecord         */

/*                                             end of dumpinfo.c           */
