/* @(#)getcode.c	1.1 6/8/92 */
/*
  Notice:  This software is being released into the public
           domain.  It may be freely used whole or in part or 
           distributed as a whole as long as this notice remains
           attached to the source code.  The author and ESRI 
           do not assume any liability or responsibility in the 
           use or distribution of this software.  Direct any 
           questions or comments via email to the author at 
           tstellhorn@esri.com.
            
                              Todd Stellhorn, SMTS
                              ESRI, Applications Development
 */

#include <stdio.h>
#include <string.h>
#include "infolib.h"

#define USAGE "usage: getcode <table> <variable> <description>"
#define DESC_ITEM "DESCRIPTION"
#define CODE_ITEM "CODE"
#define FatalError( mess ) { (void) fprintf( stderr, "%s\n", mess ); \
  (void) fprintf( stdout, "&s .INFOStatus := .false.\n" ); exit( 1 ); }

void main( argc, argv )
int argc;
char **argv;
{
      char         *InfoFilePath, *OutVariable, *Description;
      InfoFile     *FilePtr;
      InfoItemDef  *DescItemPtr, *CodeItemPtr;
      char          RetString[4096], tmpstr[4096];
      double        RetDouble;
      long int      RecordNumber = 0;

      if( argc != 4 ) FatalError( USAGE );

      InfoFilePath = argv[1]; 
      OutVariable = argv[2]; 
      Description = argv[3];

      if(( FilePtr = InfoOpenFile( InfoFilePath, InfoREAD )) 
           == ( InfoFile * ) NULL )
           FatalError( USAGE );

      if(( DescItemPtr = InfoGetItemDef( FilePtr, DESC_ITEM )) 
           == ( InfoItemDef * ) NULL )
           FatalError( 
           "Error getting DESCRIPTION item definition." );

      if(( CodeItemPtr = InfoGetItemDef( FilePtr, CODE_ITEM )) 
           == ( InfoItemDef * ) NULL )
           FatalError( "Error getting CODE item definition." );

      if( ! InfoSeqSearch( FilePtr, DescItemPtr, Description, 0, 
           &RecordNumber ))
           FatalError( "Error searching file." );

      if( ! InfoDecode( FilePtr, CodeItemPtr, RetString, 
           &RetDouble ))
           FatalError( "Decode error." );

      switch( InfoItemType( CodeItemPtr ))
      {
           case INFO_DATE_TYPE         :
           case INFO_CHARACTER_TYPE    :
                strcat( tmpstr, RetString ); break;

           case INFO_INTEGER_TYPE      :
                (void) sprintf( tmpstr, "%d", ( int ) RetDouble ); break;

           case INFO_NUMBER_TYPE       : 
                (void) sprintf( tmpstr, "%f", RetDouble ); break;

           case INFO_BINARY_TYPE       :
                (void) sprintf( tmpstr, "%d", ( int ) RetDouble ); break;

           case INFO_FLOATING_TYPE     :
                (void) sprintf( tmpstr, "%f", RetDouble ); break;

           default                     :
                FatalError( "Invalid item type." );
      }

      if( ! InfoCloseFile( FilePtr ))
           FatalError( "Error closing file." );

      (void) fprintf( stdout, "&s %s := %s\n", OutVariable, tmpstr );
      (void) fprintf( stdout, "&s .INFOStatus := .true.\n" );

}
