
#include "infolib.h"

#define USAGE "usage: infosort <table> <item1...itemn>"
#define FatalError( mess ) { fprintf( stderr, "%s\n", mess ); exit( 1 ); }
#define STRING_SIZE 4096

short int InfoSortFile();
int sortcmp();

InfoItemDef **itemlist;

int main( argc, argv )
int argc;
char **argv;
{
     if( argc <= 2 )
          FatalError( USAGE );

     if( ! InfoSortFile( argv[1], ( long int )( argc - 2 ), &argv[2] ))
          FatalError( "Error sorting file...Exiting..." );

     return( 0 );
}

short int InfoSortFile( filename, nitems, items )
char *filename;
long int nitems;
char **items;
{
     long int i, k, filesize;
     InfoFile *ip;
     char *datbuf;

     if(( itemlist = ( InfoItemDef ** )
          malloc(( sizeof( InfoItemDef * )) * ( nitems + 1 ))) == NULL )
          return( 0 );
     itemlist[nitems] = NULL;

     if(( ip = InfoOpenFile( filename, InfoWRITE )) == NULL )
     {
          free( itemlist );
          return( 0 );
     }

     filesize = ( ip->NumberRecords * ip->RecordLength ) * sizeof( char );

     if(( datbuf = ( char * )malloc( filesize )) == NULL )
     {
          free( itemlist );
          InfoCloseFile( ip );
          return( 0 );
     }

     for( i = 0; i < nitems; i++ )
     {
          if(( itemlist[i] = InfoGetItemDef( ip, items[i] )) == NULL )
          {
               free( datbuf );
               for( k = 0; k < i; k++ )
                    free( itemlist[k] );
               free( itemlist );
               InfoCloseFile( ip );
               return( 0 );
          }
     }

     if(( fread( datbuf, filesize, 1, ip->InfoFILE )) != 1 )
     {
          free( datbuf );
          for( k = 0; k < nitems; k++ )
               free( itemlist[k] );
          free( itemlist );
          InfoCloseFile( ip );
          return( 0 );
     }

     qsort( datbuf, ip->NumberRecords, ip->RecordLength, sortcmp );

     for( k = 0; k < nitems; k++ )
          free( itemlist[k] );
     free( itemlist );

     if( fseek( ip->InfoFILE, 0L, 0 ))
     {
          free( datbuf );
          InfoCloseFile( ip );
          return( 0 );
     }

     if(( fwrite( datbuf, filesize, 1, ip->InfoFILE )) != 1 )
     {
          free( datbuf );
          InfoCloseFile( ip );
          return( 0 );
     }

     InfoCloseFile( ip );
     free( datbuf );

     return( 1 );
}

int sortcmp( buf1, buf2 )
void *buf1;
void *buf2;
{
     InfoFile rec1, rec2;
     double val1, val2, cmpval = 0.0;
     char str1[STRING_SIZE], str2[STRING_SIZE];
     long int i;

     rec1.IOBuffer = ( char * )buf1;
     rec2.IOBuffer = ( char * )buf2;

     for( i = 0; itemlist[i]; i++ )
     {
          InfoDecode( &rec1, itemlist[i], str1, &val1 );
          InfoDecode( &rec2, itemlist[i], str2, &val2 );

          if( itemlist[i]->ItemType == INFO_CHARACTER_TYPE )
               cmpval = ( double )strcmp( str1, str2 );
          else
               cmpval = val1 - val2;

          if( cmpval != 0.0 )
          {
               if( cmpval < 0.0 )
                    return( -1 );
               else
                    return( 1 );
          }
     }
     return( 0 );
}
