/*mgd77 written to import MGD77 format data into ARC/INFO*/
/*written by Alan Brenner 29 August 94*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "info/infolib.h"

#define ITEMS 25
#define LENGTH 200
#define LATNUM 7
#define LONNUM 8

void error(val, mess)                                       /*error occurance*/
  int val;
  char *mess;
{
     (void) fprintf(stderr,"\n%s\n\n",mess);
     exit(val);
}

void define(outfile, name, width, output, type, decimals)   /* item definition function */
  char *outfile, *name;
  short width, output, decimals, type;
{
     char temp[120];

     if (InfoDefineItem(outfile,name,width,output,(long) type,decimals)==INFO_FAIL)
     {							    /* define item with error check */
	  (void) sprintf(temp,"Could not define %s in the output table.",name);
	  error(3,temp);
     }
}
     
int main (argc, argv)					    /*main program function*/
  int argc;
  char *argv[];
{							    /* there are ITEMS of these names */
     char *names[] = {"TYPE","CRUISE","TZ_CORRECTION","DATE","HOUR","MINUTE","LATITUDE","LONGITUDE",
			   "POSITION_TYPE","B_TRAVELTIME","B_CORR_DEPTH","B_CORR_CODE","B_TYPE_CODE",
			   "M_SENSOR_1","M_SENSOR_2","M_RESIDUAL","M_RESID_SENS","M_DIURN_CORR","M_LEAD_SENSOR",
			   "GRAVITY","EOTVOS","FREE_AIR_ANOM","SEISMIC_LINE","SEISMIC_SHOT","QUALITY_CODE"};
     char line[LENGTH+1],cdata[LENGTH];
     short width,decimals;
     int count,counted,counter,counting=0;
     double ndata=0.0,divisor,lat,lon;
     FILE *datafile,*genfile;
     InfoFile *infofile;
     InfoItemDef *pat,*items[ITEMS];


     if (argc!=3)					    /* check usage */
	  error(-1,"Usage: mgd77 <data_file> <output_table>");

     if ((datafile=fopen(argv[1],"rt"))==NULL)		    /* open input data file */
	  error(1,"Could not open data file.");

     if ((genfile=fopen("xx_mgd77arc.gen","wt"))==NULL)	    /* file for lat-long points */
	  error(1,"Could not open generate output file.");

     if (InfoCreateFile(argv[2],(InfoFile*)NULL)==INFO_FAIL)
	  error(2,"Could not create INFO table.");	    /* create info table */

     define(argv[2],"PAT-ID",4,10,INFO_BINARY_TYPE,0);
     define(argv[2],names[0],1,1,INFO_INTEGER_TYPE,0);	    /* define items */
     define(argv[2],names[1],8,8,INFO_CHARACTER_TYPE,0);    /* For character and integer */
     define(argv[2],names[2],4,6,INFO_FLOATING_TYPE,2);	    /* items output width must be */
     define(argv[2],names[3],8,8,INFO_DATE_TYPE,0);	    /* set equal to the input line*/
     define(argv[2],names[4],2,2,INFO_INTEGER_TYPE,0);	    /* width--set the internal */
     define(argv[2],names[5],4,6,INFO_FLOATING_TYPE,3);	    /* width to the same size. */
     define(argv[2],names[6],8,9,INFO_FLOATING_TYPE,5);	    /* For floats the output width */
     define(argv[2],names[7],8,10,INFO_FLOATING_TYPE,5);    /* must be one greater than */
     define(argv[2],names[8],1,1,INFO_INTEGER_TYPE,0);	    /* the input line width--the */
     define(argv[2],names[9],4,7,INFO_FLOATING_TYPE,4);	    /* decimal point is implied */
     define(argv[2],names[10],4,7,INFO_FLOATING_TYPE,1);    /* in the input, but must be */
     define(argv[2],names[11],2,2,INFO_INTEGER_TYPE,0);	    /* accounted for in INFO. */
     define(argv[2],names[12],1,1,INFO_INTEGER_TYPE,0);
     define(argv[2],names[13],4,7,INFO_FLOATING_TYPE,1);
     define(argv[2],names[14],4,7,INFO_FLOATING_TYPE,1);
     define(argv[2],names[15],4,7,INFO_FLOATING_TYPE,1);
     define(argv[2],names[16],1,1,INFO_INTEGER_TYPE,0);
     define(argv[2],names[17],4,6,INFO_FLOATING_TYPE,1);
     define(argv[2],names[18],6,6,INFO_INTEGER_TYPE,0);
     define(argv[2],names[19],4,8,INFO_FLOATING_TYPE,1);
     define(argv[2],names[20],4,7,INFO_FLOATING_TYPE,1);
     define(argv[2],names[21],4,6,INFO_FLOATING_TYPE,1);
     define(argv[2],names[22],5,5,INFO_CHARACTER_TYPE,0);
     define(argv[2],names[23],6,6,INFO_CHARACTER_TYPE,0);
     define(argv[2],names[24],1,1,INFO_INTEGER_TYPE,0);


     if ((infofile=InfoOpenFile(argv[2],InfoWRITE))==(InfoFile*)NULL)
	  error(4,"Could not open info file.");		    /* open lookup table */

     if ((pat=InfoGetItemDef(infofile,"PAT-ID"))==(InfoItemDef*)NULL)
	  error(5,"Could not open table item.");
     for (count=0; count<ITEMS; count++)
     {							    /* get each item */
	  if ((items[count]=InfoGetItemDef(infofile,names[count]))==(InfoItemDef*)NULL)
	       error(5,"Could not open table item.");
     }


     while (fgets(line,LENGTH,datafile)!=NULL)
     {
	  if (InfoEncode(infofile,pat,line,(double) ++counting)==INFO_FAIL)
	       error(6,"Error encoding data item.");
	  
	  counter=0;					    /* point to the beginning of the record */

	  for (count=0; count<ITEMS; count++)		    /* insert data into output que*/
	  {
	       (void) memset(cdata,0,LENGTH);		    /* zero out character output */

	       width=InfoItemOutputWidth(items[count]);
	       switch(InfoItemType(items[count]))
	       {
	       case INFO_CHARACTER_TYPE :		    /* copy characters to output */
		    (void) strncpy(cdata,line+counter,width);
		    counter+=width;
		    break;
	       case INFO_DATE_TYPE :			    /* copy dates to output */
		    (void) strcpy(cdata,"19");		    /* insert century */
		    (void) strncat(cdata,line+counter,6);   /* append YYMMDD */
		    counter+=6;
		    break;
	       case INFO_INTEGER_TYPE :			    /* translate integers to output */
		    (void) strncpy(cdata,line+counter,width);
		    ndata=atof(cdata);
		    counter+=width;
		    break;
	       default :				    /* translate floats to output */
		    divisor=10.0;
		    decimals=InfoItemNumberDecimals(items[count]);
		    (void) strncpy(cdata,line+counter,width-1);
		    for (counted=1; counted<decimals; counted++) divisor*=10.0;
		    ndata=atof(cdata)/divisor;
		    counter+=(InfoItemOutputWidth(items[count])-1);
		    if (count==LATNUM-1) lat=ndata;
		    if (count==LONNUM-1) lon=ndata;
		    break;
	       }
	       if (InfoEncode(infofile,items[count],cdata,ndata)==INFO_FAIL)
		    error(6,"Error encoding data item.");
	  }

	  if (InfoWriteRecord(infofile,(long) counting)==INFO_FAIL)
	       error(7,"Error writing INFO table.");
	  if (fprintf(genfile,"%d,%f,%f\n",counting,lon,lat)==EOF)
	       error(8,"Error writing generate file.");
     }

     if (InfoFileFlush(infofile)==INFO_FAIL)		    /* flush and close info file */
	  error(9,"Error flushing info file.");
     if (InfoCloseFile(infofile)==INFO_FAIL)
	  error(9,"Error closing info file.");

     if (fprintf(genfile,"end\n")==EOF)
	  error(10,"Error ending generate file.");
     if (fclose(genfile)!=0)
	  error(10,"Error closing generate file.");

     return(fclose(datafile));				    /*return no errors?*/
}
